#!/bin/bash
# -*- mode: shell-script; indent-tabs-mode: nil; sh-basic-offset: 4; -*-
# ex: ts=8 sw=4 sts=4 et filetype=sh

# called by dracut
check() {
    return 255
}

# called by dracut
depends() {
    return 0
}

# called by dracut
installkernel() {
    local _fipsmodules _mod
    _fipsmodules="aead aes_generic aes-x86_64 ansi_cprng arc4 blowfish camellia cast6 cbc ccm "
    _fipsmodules+="chainiv crc32c crct10dif_generic cryptomgr crypto_null ctr cts deflate des des3_ede dm-crypt dm-mod "
    _fipsmodules+="ecb eseqiv fcrypt gcm ghash_generic hmac khazad lzo md4 md5 michael_mic rmd128 cmac "
    _fipsmodules+="rmd160 rmd256 rmd320 salsa20 seed seqiv serpent sha1 sha224 sha256 sha256_generic "
    _fipsmodules+="sha384 sha512 sha512_generic tcrypt tea tnepres twofish wp256 wp384 wp512 xeta xtea xts zlib "
    _fipsmodules+="sha512-ssse3 sha1-ssse3 sha256-ssse3 "
    _fipsmodules+="ghash-clmulni-intel aesni-intel "

    _fipsmodules+="drbg"

    mkdir -m 0755 -p "${initdir}/etc/modprobe.d"

    for _mod in $_fipsmodules; do
        if hostonly='' instmods -c -s $_mod; then
            echo $_mod >> "${initdir}/etc/fipsmodules"
            echo "blacklist $_mod" >> "${initdir}/etc/modprobe.d/fips.conf"
        fi
    done
}

# called by dracut
install() {
    local _dir
    inst_hook pre-trigger 01 "$moddir/fips-boot.sh"
    inst_hook pre-pivot 01 "$moddir/fips-noboot.sh"
    inst_script "$moddir/fips.sh" /sbin/fips.sh

    inst_multiple rmmod insmod mount uname umount fipscheck strace

    inst_libdir_file \
        fipscheck .fipscheck.hmac \
         libfipscheck.so.1 \
        .libfipscheck.so.1.hmac .libfipscheck.so.1.1.0.hmac \
         libcrypto.so.1.0.0       libssl.so.1.0.0 \
        .libcrypto.so.1.0.0.hmac .libssl.so.1.0.0.hmac \
        .libcryptsetup.so.4.5.0.hmac .libcryptsetup.so.4.hmac \
        .libgcrypt.so.20.hmac \

    # we do not use prelink at SUSE
    #inst_multiple -o prelink

    inst_simple /etc/system-fips
}

